@@ -50,6 +50,33 @@ page(s), you can pass the ``--show`` option on the command line:
 This will open two pages, for ``/app_script`` and ``/app_dir``,
 respectively.
 
+If you would like to pass command line arguments to Bokeh applications,
+you can pass the ``--args`` option as the LAST option on the command
+line:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py myapp.py --args foo bar --baz
+
+Everything that follows ``--args`` will be included in ``sys.argv`` when
+the application runs. In this case, when ``myapp.py`` executes, the
+contents of ``sys.argv`` will be ``['myapp.py', 'foo', 'bar', '--baz']``,
+consistent with standard Python expectations for ``sys.argv``.
+
+Note that if multiple scripts or directories are provided, they
+all receive the same set of command line arguments (if any) given by
+``--args``.
+
+If you have only one application, the server root will redirect to it.
+Otherwise, You can see an index of all running applications at the server root:
+
+.. code-block:: none
+
+    http://localhost:5006/
+
+This index can be disabled with the ``--disable-index`` option, and the redirect
+behavior can be disabled with the ``--disable-index-redirect`` option.
+
 Network Configuration
 ~~~~~~~~~~~~~~~~~~~~~
 
@@ -58,35 +85,33 @@ argument:
 
 .. code-block:: sh
 
-    bokeh serve app_script.py --port=8080
+    bokeh serve app_script.py --port 8080
+
+To listen on an arbitrary port, pass ``0`` as the port number.  The actual
+port number will be logged at startup.
 
 Similarly, a specific network address can be specified with the
 ``--address`` argument. For example:
 
 .. code-block:: sh
 
-    bokeh serve app_script.py --address=0.0.0.0
+    bokeh serve app_script.py --address 0.0.0.0
 
 will have the Bokeh server listen all available network addresses.
 
-Additionally, it is possible to configure a hosts whitelist that must be
-matched by the ``Host`` header in new requests. You can specify multiple
-acceptable host values with the ``--host`` option:
+Bokeh server can fork the underlying tornado server into multiprocess.  This is
+useful when trying to handle multiple connections especially in the context of
+apps which require high computational loads.  Default behavior is one process.
+using 0 will auto-detect the number of cores and spin up corresponding number of
+processes
 
 .. code-block:: sh
 
-    bokeh serve app_script.py --host foo.com:8081 --host bar.com
+    bokeh serve app_script.py --num-procs 2
 
-If no port is specified in a host value, then port 80 will be used. In
-the example above Bokeh server will accept requests from ``foo.com:8081``
-and ``bar.com:80``.
-
-If no host values are specified, then by default the Bokeh server will
-accept requests from ``localhost:<port>`` where ``<port>`` is the port
-that the server is configured to listen on (by default: {DEFAULT_PORT}).
-
-Also note that the host whitelist applies to all request handlers,
-including any extra ones added to extend the Bokeh server.
+Note that due to limitations inherent in Tornado, Windows does not support
+``--num-procs`` values greater than one! In this case consider running multiple
+Bokeh server instances behind a load balancer.
 
 By default, cross site connections to the Bokeh server websocket are not
 allowed. You can enable websocket connections originating from additional
@@ -104,7 +129,7 @@ This can often be useful in conjunction with "reverse proxy" setups.
 
 .. code-block:: sh
 
-    bokeh serve app_script.py --prefix=foobar
+    bokeh serve app_script.py --prefix foobar
 
 Then the application will be served under the following URL:
 
@@ -123,7 +148,7 @@ The value is specified in milliseconds. The default keep-alive interval
 is 37 seconds. Give a value of 0 to disable keep-alive pings.
 
 To control how often statistic logs are written, set the
---stats-log-frequency option:
+``--stats-log-frequency`` option:
 
 .. code-block:: sh
 
@@ -132,6 +157,17 @@ To control how often statistic logs are written, set the
 The value is specified in milliseconds. The default interval for
 logging stats is 15 seconds. Only positive integer values are accepted.
 
+Bokeh can also optionally log process memory usage. This feature requires
+the optional ``psutil`` package to be installed. To enable memory logging
+set the ``--mem-log-frequency`` option:
+
+. code-block:: sh
+
+    bokeh serve app_script.py --mem-log-frequency 30000
+
+The value is specified in milliseconds. The default interval for
+logging stats is 0 (disabled). Only positive integer values are accepted.
+
 To have the Bokeh server override the remote IP and URI scheme/protocol for
 all requests with ``X-Real-Ip``, ``X-Forwarded-For``, ``X-Scheme``,
 ``X-Forwarded-Proto``  headers (if they are provided), set the
@@ -163,7 +199,7 @@ argument:
 
 .. code-block:: sh
 
-    bokeh serve app_script.py --session-ids=signed
+    bokeh serve app_script.py --session-ids signed
 
 The available modes are: {SESSION_ID_MODES}
 
@@ -236,42 +272,102 @@ To configure how often unused sessions last. set the
     bokeh serve app_script.py --unused-session-lifetime 60000
 
 The value is specified in milliseconds. The default lifetime interval
-for unused sessions is 30 minutes. Only positive integer values are
+for unused sessions is 15 seconds. Only positive integer values are
 accepted.
 
-Development Options
-~~~~~~~~~~~~~~~~~~~
+Logging Options
+~~~~~~~~~~~~~~~
 
 The logging level can be controlled by the ``--log-level`` argument:
 
 .. code-block:: sh
 
-    bokeh serve app_script.py --log-level=debug
+    bokeh serve app_script.py --log-level debug
 
 The available log levels are: {LOGLEVELS}
 
+The log format can be controlled by the ``--log-format`` argument:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --log-format "%(levelname)s: %(message)s"
+
+The default log format is ``"{DEFAULT_LOG_FORMAT}"``
+
 '''
 from __future__ import absolute_import
 
 import logging
 log = logging.getLogger(__name__)
 
+import argparse
+import warnings
+
 from bokeh.application import Application
 from bokeh.resources import DEFAULT_SERVER_PORT
-from bokeh.server.server import Server
-from bokeh.util.string import nice_join
+from bokeh.util.logconfig import basicConfig
+from bokeh.util.string import nice_join, format_docstring
 from bokeh.settings import settings
 
+from os import getpid
+
 from ..subcommand import Subcommand
-from ..util import build_single_handler_applications, die
+from ..util import build_single_handler_applications, die, report_server_init_errors
 
-LOGLEVELS = ('debug', 'info', 'warning', 'error', 'critical')
+LOGLEVELS = ('trace', 'debug', 'info', 'warning', 'error', 'critical')
 SESSION_ID_MODES = ('unsigned', 'signed', 'external-signed')
+DEFAULT_LOG_FORMAT = "%(asctime)s %(message)s"
 
-__doc__ = __doc__.format(
+__doc__ = format_docstring(__doc__,
     DEFAULT_PORT=DEFAULT_SERVER_PORT,
     LOGLEVELS=nice_join(LOGLEVELS),
-    SESSION_ID_MODES=nice_join(SESSION_ID_MODES)
+    SESSION_ID_MODES=nice_join(SESSION_ID_MODES),
+    DEFAULT_LOG_FORMAT=DEFAULT_LOG_FORMAT
+)
+
+def _fixup_deprecated_host_args(args):
+    if args.host is not None and len(args.host) > 0:
+        if args.allow_websocket_origin is None:
+            args.allow_websocket_origin = []
+        args.allow_websocket_origin += args.host
+        args.allow_websocket_origin = list(set(args.allow_websocket_origin))
+        warnings.warn(
+            "The --host parameter is deprecated because it is no longer needed. "
+            "It will be removed and trigger an error in a future release. "
+            "Values set now will be copied to --allow-websocket-origin. "
+            "Depending on your use case, you may need to set current --host "
+            "values for 'allow_websocket_origin' instead."
+        )
+
+base_serve_args = (
+    ('--port', dict(
+        metavar = 'PORT',
+        type    = int,
+        help    = "Port to listen on",
+        default = DEFAULT_SERVER_PORT
+    )),
+
+    ('--address', dict(
+        metavar = 'ADDRESS',
+        type    = str,
+        help    = "Address to listen on",
+        default = None,
+    )),
+
+    ('--log-level', dict(
+        metavar = 'LOG-LEVEL',
+        action  = 'store',
+        default = 'info',
+        choices = LOGLEVELS,
+        help    = "One of: %s" % nice_join(LOGLEVELS),
+    )),
+
+    ('--log-format', dict(
+        metavar ='LOG-FORMAT',
+        action  = 'store',
+        default = DEFAULT_LOG_FORMAT,
+        help    = "A standard Python logging format string (default: %r)" % DEFAULT_LOG_FORMAT.replace("%", "%%"),
+    )),
 )
 
 class Serve(Subcommand):
@@ -279,12 +375,12 @@ class Serve(Subcommand):
 
     '''
 
+    #: name for this subcommand
     name = "serve"
 
     help = "Run a Bokeh server hosting one or more applications"
 
-    args = (
-
+    args = base_serve_args + (
         ('files', dict(
             metavar='DIRECTORY-OR-SCRIPT',
             nargs='*',
@@ -292,9 +388,10 @@ class Serve(Subcommand):
             default=None,
         )),
 
-        ('--develop', dict(
-            action='store_true',
-            help="Enable develop-time features that should not be used in production",
+        ('--args', dict(
+            metavar='COMMAND-LINE-ARGS',
+            nargs=argparse.REMAINDER,
+            help="Any command line arguments remaining are passed on to the application handler",
         )),
 
         ('--show', dict(
@@ -302,20 +399,6 @@ class Serve(Subcommand):
             help="Open server app(s) in a browser",
         )),
 
-        ('--port', dict(
-            metavar='PORT',
-            type=int,
-            help="Port to listen on",
-            default=None
-        )),
-
-        ('--address', dict(
-            metavar='ADDRESS',
-            type=str,
-            help="Address to listen on",
-            default=None,
-        )),
-
         ('--allow-websocket-origin', dict(
             metavar='HOST[:PORT]',
             action='append',
@@ -327,7 +410,7 @@ class Serve(Subcommand):
             metavar='HOST[:PORT]',
             action='append',
             type=str,
-            help="Public hostnames to allow in requests",
+            help="*** DEPRECATED ***",
         )),
 
         ('--prefix', dict(
@@ -365,19 +448,18 @@ class Serve(Subcommand):
             default=None,
         )),
 
+        ('--mem-log-frequency', dict(
+            metavar='MILLISECONDS',
+            type=int,
+            help="How often to log memory usage information",
+            default=None,
+        )),
+
         ('--use-xheaders', dict(
             action='store_true',
             help="Prefer X-headers for IP/protocol information",
         )),
 
-        ('--log-level', dict(
-            metavar='LOG-LEVEL',
-            action  = 'store',
-            default = 'debug',
-            choices = LOGLEVELS,
-            help    = "One of: %s" % nice_join(LOGLEVELS),
-        )),
-
         ('--session-ids', dict(
             metavar='MODE',
             action  = 'store',
@@ -386,51 +468,75 @@ class Serve(Subcommand):
             help    = "One of: %s" % nice_join(SESSION_ID_MODES),
         )),
 
+        ('--disable-index', dict(
+            action = 'store_true',
+            help    = 'Do not use the default index on the root path',
+        )),
+
+        ('--disable-index-redirect', dict(
+            action = 'store_true',
+            help    = 'Do not redirect to running app from root path',
+        )),
+
+        ('--num-procs', dict(
+            metavar='N',
+            action='store',
+            help="Number of worker processes for an app. Default to one. Using "
+                 "0 will autodetect number of cores",
+            default=1,
+            type=int,
+        )),
     )
 
+
     def invoke(self, args):
-        applications = build_single_handler_applications(args.files)
+        '''
+
+        '''
+
+        # protect this import inside a function so that "bokeh info" can work
+        # even if Tornado is not installed
+        from bokeh.server.server import Server
+
+        argvs = { f : args.args for f in args.files}
+        applications = build_single_handler_applications(args.files, argvs)
 
         log_level = getattr(logging, args.log_level.upper())
-        logging.basicConfig(level=log_level)
+        basicConfig(level=log_level, format=args.log_format)
+
+        # This should remain here until --host is removed entirely
+        _fixup_deprecated_host_args(args)
 
         if len(applications) == 0:
-            # create an empty application by default, typically used with output_server
+            # create an empty application by default
             applications['/'] = Application()
 
+        # rename args to be compatible with Server
         if args.keep_alive is not None:
-            if args.keep_alive == 0:
-                log.info("Keep-alive ping disabled")
-            else:
-                log.info("Keep-alive ping configured every %d milliseconds", args.keep_alive)
-            # rename to be compatible with Server
             args.keep_alive_milliseconds = args.keep_alive
 
         if args.check_unused_sessions is not None:
-            log.info("Check for unused sessions every %d milliseconds", args.check_unused_sessions)
-            # rename to be compatible with Server
             args.check_unused_sessions_milliseconds = args.check_unused_sessions
 
         if args.unused_session_lifetime is not None:
-            log.info("Unused sessions last for %d milliseconds", args.unused_session_lifetime)
-            # rename to be compatible with Server
             args.unused_session_lifetime_milliseconds = args.unused_session_lifetime
 
         if args.stats_log_frequency is not None:
-            log.info("Log statistics every %d milliseconds", args.stats_log_frequency)
-            # rename to be compatible with Server
             args.stats_log_frequency_milliseconds = args.stats_log_frequency
 
+        if args.mem_log_frequency is not None:
+            args.mem_log_frequency_milliseconds = args.mem_log_frequency
+
         server_kwargs = { key: getattr(args, key) for key in ['port',
                                                               'address',
                                                               'allow_websocket_origin',
-                                                              'host',
+                                                              'num_procs',
                                                               'prefix',
-                                                              'develop',
                                                               'keep_alive_milliseconds',
                                                               'check_unused_sessions_milliseconds',
                                                               'unused_session_lifetime_milliseconds',
                                                               'stats_log_frequency_milliseconds',
+                                                              'mem_log_frequency_milliseconds',
                                                               'use_xheaders',
                                                             ]
                           if getattr(args, key, None) is not None }
@@ -456,25 +562,26 @@ class Serve(Subcommand):
             die("To sign sessions, the BOKEH_SECRET_KEY environment variable must be set; " +
                 "the `bokeh secret` command can be used to generate a new key.")
 
-        server = Server(applications, **server_kwargs)
+        server_kwargs['use_index'] = not args.disable_index
+        server_kwargs['redirect_root'] = not args.disable_index_redirect
 
-        if args.show:
-            # we have to defer opening in browser until we start up the server
-            def show_callback():
-                for route in applications.keys():
-                    server.show(route)
-            server.io_loop.add_callback(show_callback)
+        with report_server_init_errors(**server_kwargs):
+            server = Server(applications, **server_kwargs)
 
-        if args.develop:
-            log.info("Using develop mode (do not enable --develop in production)")
+            if args.show:
+                # we have to defer opening in browser until we start up the server
+                def show_callback():
+                    for route in applications.keys():
+                        server.show(route)
+                server.io_loop.add_callback(show_callback)
 
-        address_string = ''
-        if server.address is not None and server.address != '':
-            address_string = ' address ' + server.address
+            address_string = 'localhost'
+            if server.address is not None and server.address != '':
+                address_string = server.address
 
-        log.info("Starting Bokeh server on port %d%s with applications at paths %r",
-                 server.port,
-                 address_string,
-                 sorted(applications.keys()))
+            for route in sorted(applications.keys()):
+                url = "http://%s:%d%s%s" % (address_string, server.port, server.prefix, route)
+                log.info("Bokeh app running at: %s" % url)
 
-        server.start()
+            log.info("Starting Bokeh server with process id: %d" % getpid())
+            server.run_until_shutdown()
