@@ -1,30 +1,70 @@
 # -*- coding: utf-8 -*-
-""" Models for display visual shapes whose attributes can be associated
-with data columns from data sources.
+''' Display a variety of visual shapes whose attributes can be associated
+with data columns from ``ColumnDataSources``.
 
-"""
+The full list of glyphs built into Bokeh is given below:
+
+* :class:`~bokeh.models.glyphs.AnnularWedge`
+* :class:`~bokeh.models.glyphs.Annulus`
+* :class:`~bokeh.models.glyphs.Arc`
+* :class:`~bokeh.models.glyphs.Bezier`
+* :class:`~bokeh.models.glyphs.Ellipse`
+* :class:`~bokeh.models.glyphs.HBar`
+* :class:`~bokeh.models.glyphs.HexTile`
+* :class:`~bokeh.models.glyphs.Image`
+* :class:`~bokeh.models.glyphs.ImageRGBA`
+* :class:`~bokeh.models.glyphs.ImageURL`
+* :class:`~bokeh.models.glyphs.Line`
+* :class:`~bokeh.models.glyphs.MultiLine`
+* :class:`~bokeh.models.glyphs.Oval`
+* :class:`~bokeh.models.glyphs.Patch`
+* :class:`~bokeh.models.glyphs.Patches`
+* :class:`~bokeh.models.glyphs.Quad`
+* :class:`~bokeh.models.glyphs.Quadratic`
+* :class:`~bokeh.models.glyphs.Ray`
+* :class:`~bokeh.models.glyphs.Rect`
+* :class:`~bokeh.models.glyphs.Segment`
+* :class:`~bokeh.models.glyphs.Step`
+* :class:`~bokeh.models.glyphs.Text`
+* :class:`~bokeh.models.glyphs.VBar`
+* :class:`~bokeh.models.glyphs.Wedge`
+
+All these glyphs share a minimal common interface through their base class
+``Glyph``:
+
+.. autoclass:: Glyph
+    :members:
+
+'''
 from __future__ import absolute_import
 
-from ..core.enums import Direction, Anchor
-from ..core.property_mixins import FillProps, LineProps, TextProps
+from ..core.enums import Anchor, Direction, StepMode
+from ..core.has_props import abstract
+from ..core.properties import (AngleSpec, Bool, DistanceSpec, Enum, Float, String,
+                               Include, Instance, Int, NumberSpec, Override, StringSpec)
+from ..core.property_mixins import FillProps, LineProps, ScalarFillProps, ScalarLineProps, TextProps
 from ..model import Model
-from ..core.properties import (abstract, AngleSpec, Bool, DistanceSpec, Enum, Float,
-                          Include, Instance, Int, NumberSpec, StringSpec)
 
 from .mappers import ColorMapper, LinearColorMapper
 
 @abstract
 class Glyph(Model):
-    """ Base class for all glyph models. """
+    ''' Base class for all glyph models.
 
-    visible = Bool(True, help="""
-    Whether the glyph should render or not.
-    """)
+    '''
 
-class AnnularWedge(Glyph):
-    """ Render annular wedges. """
+@abstract
+class XYGlyph(Glyph):
+    ''' Base class of glyphs with `x` and `y` coordinate attributes.
 
-    __example__ = "tests/glyphs/AnnularWedge.py"
+    '''
+
+class AnnularWedge(XYGlyph):
+    ''' Render annular wedges.
+
+    '''
+
+    __example__ = "examples/reference/models/AnnularWedge.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -66,10 +106,12 @@ class AnnularWedge(Glyph):
     The %s values for the annular wedges.
     """)
 
-class Annulus(Glyph):
-    """ Render annuli. """
+class Annulus(XYGlyph):
+    ''' Render annuli.
 
-    __example__ = "tests/glyphs/Annulus.py"
+    '''
+
+    __example__ = "examples/reference/models/Annulus.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -99,10 +141,12 @@ class Annulus(Glyph):
     The %s values for the annuli.
     """)
 
-class Arc(Glyph):
-    """ Render arcs. """
+class Arc(XYGlyph):
+    ''' Render arcs.
 
-    __example__ = "tests/glyphs/Arc.py"
+    '''
+
+    __example__ = "examples/reference/models/Arc.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -137,14 +181,15 @@ class Arc(Glyph):
     """)
 
 class Bezier(Glyph):
-    u""" Render Bézier curves.
+    u''' Render Bézier curves.
 
     For more information consult the `Wikipedia article for Bézier curve`_.
 
     .. _Wikipedia article for Bézier curve: http://en.wikipedia.org/wiki/Bézier_curve
-    """
 
-    __example__ = "tests/glyphs/Bezier.py"
+    '''
+
+    __example__ = "examples/reference/models/Bezier.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -186,10 +231,12 @@ class Bezier(Glyph):
     The %s values for the Bézier curves.
     """)
 
-class Ellipse(Glyph):
-    u""" Render ellipses. """
+class Ellipse(XYGlyph):
+    u''' Render ellipses.
 
-    __example__ = "tests/glyphs/Ellipse.py"
+    '''
+
+    __example__ = "examples/reference/models/Ellipse.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -223,81 +270,17 @@ class Ellipse(Glyph):
     The %s values for the ovals.
     """)
 
-class Gear(Glyph):
-    """ Render gears.
-
-    The details and nomenclature concerning gear construction can
-    be quite involved. For more information, consult the `Wikipedia
-    article for Gear`_.
-
-    .. _Wikipedia article for Gear: http://en.wikipedia.org/wiki/Gear
-    """
-
-    __example__ = "tests/glyphs/Gear.py"
-
-    # a canonical order for positional args that can be used for any
-    # functions derived from this class
-    _args = ('x', 'y', 'angle', 'module', 'teeth', 'pressure_angle', 'shaft_size', 'internal')
-
-    x = NumberSpec(help="""
-    The x-coordinates of the center of the gears.
-    """)
-
-    y = NumberSpec(help="""
-    The y-coordinates of the center of the gears.
-    """)
-
-    angle = AngleSpec(default=0, help="""
-    The angle the gears are rotated from horizontal. [rad]
-    """)
-
-    module = NumberSpec(help="""
-    A scaling factor, given by::
-
-        m = p / pi
-
-    where *p* is the circular pitch, defined as the distance from one
-    face of a tooth to the corresponding face of an adjacent tooth on
-    the same gear, measured along the pitch circle. [float]
-    """)
-
-    teeth = NumberSpec(help="""
-    How many teeth the gears have. [int]
-    """)
-
-    pressure_angle = NumberSpec(default=20, help="""
-    The complement of the angle between the direction that the teeth
-    exert force on each other, and the line joining the centers of the
-    two gears. [deg]
-    """)
-
-    # TODO: (bev) evidently missing a test for default value
-    shaft_size = NumberSpec(default=0.3, help="""
-    The central gear shaft size as a percentage of the overall gear
-    size. [float]
-    """)
-
-    # TODO: (bev) evidently missing a test for default value
-    internal = NumberSpec(default=False, help="""
-    Whether the gear teeth are internal. [bool]
-    """)
-
-    line_props = Include(LineProps, use_prefix=False, help="""
-    The %s values for the gears.
-    """)
-
-    fill_props = Include(FillProps, use_prefix=False, help="""
-    The %s values for the gears.
-    """)
-
 class HBar(Glyph):
-    """ Render horizontal bars, given a center coordinate, height and (left, right) coordinates. """
+    ''' Render horizontal bars, given a center coordinate, ``height`` and
+    (``left``, ``right``) coordinates.
 
-    __example__ = "tests/glyphs/HBar.py"
+    '''
+
+    __example__ = "examples/reference/models/HBar.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('y', 'height', 'left', 'right')
+    _args = ('y', 'height', 'right', 'left')
 
     y = NumberSpec(help="""
     The y-coordinates of the centers of the horizontal bars.
@@ -323,8 +306,73 @@ class HBar(Glyph):
     The %s values for the horizontal bars.
     """)
 
-class Image(Glyph):
-    """ Render images given as scalar data together with a color mapper. """
+class HexTile(Glyph):
+    ''' Render horizontal tiles on a regular hexagonal grid.
+
+    '''
+
+    __example__ = "examples/reference/models/HexTile.py"
+
+    # a canonical order for positional args that can be used for any
+    # functions derived from this class
+    _args = ('q', 'r')
+
+    size = Float(1.0, help="""
+    The radius (in data space units) of the hex tiling.
+
+    The radius is always measured along the cartesian y-axis for "pointy_top"
+    orientation, and along the cartesian x-axis for "flat_top" orientation. If
+    the aspect ratio of the underlying cartesian system is not 1-1, then the
+    tiles may be "squished" in one direction. To ensure that the tiles are
+    always regular hexagons, consider setting the ``match_aspect`` property of
+    the plot to True.
+    """)
+
+    aspect_scale = Float(1.0, help="""
+
+    """)
+
+    r = NumberSpec(help="""
+    The "row" axial coordinates of the tile centers.
+    """)
+
+    q = NumberSpec(help="""
+    The "column" axial coordinates of the tile centers.
+    """)
+
+    scale = NumberSpec(1.0, help="""
+    A scale factor for individual tiles.
+    """)
+
+    orientation = String("pointytop", help="""
+
+    """)
+
+    line_props = Include(LineProps, use_prefix=False, help="""
+    The %s values for the horizontal bars.
+    """)
+
+    line_color = Override(default=None)
+
+    fill_props = Include(FillProps, use_prefix=False, help="""
+    The %s values for the horizontal bars.
+    """)
+
+
+class Image(XYGlyph):
+    ''' Render images given as scalar data together with a color mapper.
+
+    In addition to the defined model properties, ``Image`` also can accept
+    a keyword argument ``palette`` in place of an explicit ``color_mapper``.
+    The value should be a list of colors, or the name of one of the built-in
+    palettes in ``bokeh.palettes``. This palette will be used to automatically
+    construct a ``ColorMapper`` model for the ``color_mapper`` property.
+
+    If both ``palette`` and ``color_mapper`` are passed, a ``ValueError``
+    exception will be raised. If neither is passed, then the ``Greys9``
+    palette will be used as a default.
+
+    '''
 
     def __init__(self, **kwargs):
         if 'palette' in kwargs and 'color_mapper' in kwargs:
@@ -339,7 +387,15 @@ class Image(Glyph):
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('image', 'x', 'y', 'dw', 'dh', 'dilate')
+    _args = ('image', 'x', 'y', 'dw', 'dh', 'global_alpha', 'dilate')
+
+    # a hook to specify any additional kwargs handled by an initializer
+    _extra_kws = {
+        'palette': (
+            'str or list[color value]',
+            'a palette to construct a value for the color mapper property from'
+        )
+    }
 
     image = NumberSpec(help="""
     The arrays of scalar data for the images to be colormapped.
@@ -369,6 +425,11 @@ class Image(Glyph):
         That number is fixed by the image itself.
     """)
 
+    global_alpha = Float(1.0, help="""
+    An overall opacity that each image is rendered with (in addition
+    to any alpha values applied explicitly in a color mapper).
+    """)
+
     dilate = Bool(False, help="""
     Whether to always round fractional pixel locations in such a way
     as to make the images bigger.
@@ -388,12 +449,14 @@ class Image(Glyph):
     # TODO: (bev) support anchor property for Image
     # ref: https://github.com/bokeh/bokeh/issues/1763
 
-class ImageRGBA(Glyph):
-    """ Render images given as RGBA data. """
+class ImageRGBA(XYGlyph):
+    ''' Render images given as RGBA data.
+
+    '''
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('image', 'x', 'y', 'dw', 'dh', 'dilate')
+    _args = ('image', 'x', 'y', 'dw', 'dh', 'global_alpha', 'dilate')
 
     image = NumberSpec(help="""
     The arrays of RGBA data for the images.
@@ -407,14 +470,6 @@ class ImageRGBA(Glyph):
     The y-coordinates to locate the image anchors.
     """)
 
-    rows = NumberSpec(None, help="""
-    The numbers of rows in the images
-    """)
-
-    cols = NumberSpec(None, help="""
-    The numbers of columns in the images
-    """)
-
     dw = DistanceSpec(help="""
     The widths of the plot regions that the images will occupy.
 
@@ -431,6 +486,11 @@ class ImageRGBA(Glyph):
         That number is fixed by the image itself.
     """)
 
+    global_alpha = Float(1.0, help="""
+    An overall opacity that each image is rendered with (in addition
+    to any inherent alpha values in the image itself).
+    """)
+
     dilate = Bool(False, help="""
     Whether to always round fractional pixel locations in such a way
     as to make the images bigger.
@@ -442,16 +502,19 @@ class ImageRGBA(Glyph):
     # TODO: (bev) support anchor property for ImageRGBA
     # ref: https://github.com/bokeh/bokeh/issues/1763
 
-class ImageURL(Glyph):
-    """ Render images loaded from given URLs. """
+class ImageURL(XYGlyph):
+    ''' Render images loaded from given URLs.
 
-    __example__ = "tests/glyphs/ImageURL.py"
+    '''
+
+    __example__ = "examples/reference/models/ImageURL.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
     _args = ('url', 'x', 'y', 'w', 'h', 'angle', 'global_alpha', 'dilate')
 
-    url = NumberSpec(help="""
+    # TODO (bev) Why is this a NumberSpec??
+    url = NumberSpec(accept_datetime=False, help="""
     The URLs to retrieve images from.
 
     .. note::
@@ -467,28 +530,18 @@ class ImageURL(Glyph):
     The y-coordinates to locate the image anchors.
     """)
 
-    # TODO: (bev) rename to "dw" for consistency
-    w = DistanceSpec(help="""
-    The widths of the plot regions that the images will occupy.
+    w = DistanceSpec(default=None, help="""
+    The height of the plot region that the image will occupy in data space.
 
-    .. note::
-        This is not the number of pixels that an image is wide.
-        That number is fixed by the image itself.
-
-    .. note::
-        This may be renamed to "dw" in the future.
+    The default value is ``None``, in which case the image will be displayed
+    at its actual image size (regardless of the units specified here).
     """)
 
-    # TODO: (bev) rename to "dh" for consistency
-    h = DistanceSpec(help="""
-    The height of the plot region that the image will occupy.
+    h = DistanceSpec(default=None, help="""
+    The height of the plot region that the image will occupy in data space.
 
-    .. note::
-        This is not the number of pixels that an image is tall.
-        That number is fixed by the image itself.
-
-    .. note::
-        This may be renamed to "dh" in the future.
+    The default value is ``None``, in which case the image will be displayed
+    at its actual image size (regardless of the units specified here).
     """)
 
     angle = AngleSpec(default=0, help="""
@@ -523,18 +576,18 @@ class ImageURL(Glyph):
     specified URL. Default is zero ms.
     """)
 
-class Line(Glyph):
-    """ Render a single line.
+class Line(XYGlyph):
+    ''' Render a single line.
 
-    .. note::
-        The ``Line`` glyph is different from most other glyphs in that
-        the vector of values only produces one glyph on the Plot.
-    """
+    The ``Line`` glyph is different from most other glyphs in that the vector
+    of values only produces one glyph on the Plot.
+
+    '''
     # a canonical order for positional args that can be used for any
     # functions derived from this class
     _args = ('x', 'y')
 
-    __example__ = "tests/glyphs/Line.py"
+    __example__ = "examples/reference/models/Line.py"
 
     x = NumberSpec(help="""
     The x-coordinates for the points of the line.
@@ -544,20 +597,19 @@ class Line(Glyph):
     The y-coordinates for the points of the line.
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the line.
     """)
 
 class MultiLine(Glyph):
-    """ Render several lines.
+    ''' Render several lines.
 
-    .. note::
-        The data for the ``MultiLine`` glyph is different in that the
-        vector of values is not a vector of scalars. Rather, it is a
-        "list of lists".
-    """
+    The data for the ``MultiLine`` glyph is different in that the vector of
+    values is not a vector of scalars. Rather, it is a "list of lists".
 
-    __example__ = "tests/glyphs/MultiLine.py"
+    '''
+
+    __example__ = "examples/reference/models/MultiLine.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -568,23 +620,23 @@ class MultiLine(Glyph):
     """)
 
     ys = NumberSpec(help="""
-    The x-coordinates for all the lines, given as a "list of lists".
+    The y-coordinates for all the lines, given as a "list of lists".
     """)
 
     line_props = Include(LineProps, use_prefix=False, help="""
     The %s values for the lines.
     """)
 
-class Oval(Glyph):
-    u""" Render ovals.
+class Oval(XYGlyph):
+    u''' Render ovals.
 
-    .. note::
-        This glyph renders ovals using Bézier curves, which are similar,
-        but not identical to ellipses. In particular, widths equal to heights
-        will not render circles. Use the ``Ellipse`` glyph for that.
-    """
+    This glyph renders ovals using Bézier curves, which are similar,
+    but not identical to ellipses. In particular, widths equal to heights
+    will not render circles. Use the ``Ellipse`` glyph for that.
 
-    __example__ = "tests/glyphs/Oval.py"
+    '''
+
+    __example__ = "examples/reference/models/Oval.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -618,15 +670,15 @@ class Oval(Glyph):
     The %s values for the ovals.
     """)
 
-class Patch(Glyph):
-    """ Render a single patch.
+class Patch(XYGlyph):
+    ''' Render a single patch.
 
-    .. note::
-        The ``Patch`` glyph is different from most other glyphs in that
-        the vector of values only produces one glyph on the Plot.
-    """
+    The ``Patch`` glyph is different from most other glyphs in that the vector
+    of values only produces one glyph on the Plot.
 
-    __example__ = "tests/glyphs/Patch.py"
+    '''
+
+    __example__ = "examples/reference/models/Patch.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -650,24 +702,23 @@ class Patch(Glyph):
         values in the sequence.
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help="""
+    line_props = Include(ScalarLineProps, use_prefix=False, help="""
     The %s values for the patch.
     """)
 
-    fill_props = Include(FillProps, use_prefix=False, help="""
+    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
     The %s values for the patch.
     """)
 
 class Patches(Glyph):
-    """ Render several patches.
+    ''' Render several patches.
 
-    .. note::
-        The data for the ``Patches`` glyph is different in that the
-        vector of values is not a vector of scalars. Rather, it is a
-        "list of lists".
-    """
+    The data for the ``Patches`` glyph is different in that the vector of
+    values is not a vector of scalars. Rather, it is a "list of lists".
 
-    __example__ = "tests/glyphs/Patches.py"
+    '''
+
+    __example__ = "examples/reference/models/Patches.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -700,9 +751,11 @@ class Patches(Glyph):
     """)
 
 class Quad(Glyph):
-    """ Render axis-aligned quads. """
+    ''' Render axis-aligned quads.
 
-    __example__ = "tests/glyphs/Quad.py"
+    '''
+
+    __example__ = "examples/reference/models/Quad.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -733,9 +786,11 @@ class Quad(Glyph):
     """)
 
 class Quadratic(Glyph):
-    """ Render parabolas. """
+    ''' Render parabolas.
 
-    __example__ = "tests/glyphs/Quadratic.py"
+    '''
+
+    __example__ = "examples/reference/models/Quadratic.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -769,10 +824,12 @@ class Quadratic(Glyph):
     The %s values for the parabolas.
     """)
 
-class Ray(Glyph):
-    """ Render rays. """
+class Ray(XYGlyph):
+    ''' Render rays.
 
-    __example__ = "tests/glyphs/Ray.py"
+    '''
+
+    __example__ = "examples/reference/models/Ray.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -792,17 +849,19 @@ class Ray(Glyph):
 
     length = DistanceSpec(help="""
     The length to extend the ray. Note that this ``length`` defaults
-    to screen units.
+    to data units (measured in the x-direction).
     """)
 
     line_props = Include(LineProps, use_prefix=False, help="""
     The %s values for the rays.
     """)
 
-class Rect(Glyph):
-    """ Render rectangles. """
+class Rect(XYGlyph):
+    ''' Render rectangles.
 
-    __example__ = "tests/glyphs/Rect.py"
+    '''
+
+    __example__ = "examples/reference/models/Rect.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -846,9 +905,11 @@ class Rect(Glyph):
     """)
 
 class Segment(Glyph):
-    """ Render segments. """
+    ''' Render segments.
 
-    __example__ = "tests/glyphs/Segment.py"
+    '''
+
+    __example__ = "examples/reference/models/Segment.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -874,10 +935,50 @@ class Segment(Glyph):
     The %s values for the segments.
     """)
 
-class Text(Glyph):
-    """ Render text. """
+class Step(XYGlyph):
+    ''' Render step lines.
 
-    __example__ = "tests/glyphs/Text.py"
+    Step levels can be draw before, after, or centered on each point, according
+    to the value of the ``mode`` property.
+
+    The x-coordinates are assumed to be (and must be) sorted in ascending order
+    for steps to be properly rendered.
+
+    '''
+
+    __example__ = "examples/reference/models/Step.py"
+
+    # a canonical order for positional args that can be used for any
+    # functions derived from this class
+    _args = ('x', 'y')
+
+    x = NumberSpec(help="""
+    The x-coordinates for the steps.
+    """)
+
+    y = NumberSpec(help="""
+    The y-coordinates for the steps.
+    """)
+
+    line_props = Include(LineProps, use_prefix=False, help="""
+    The %s values for the steps.
+    """)
+
+    mode = Enum(StepMode, default="before", help="""
+    Where the step "level" should be drawn in relation to the x and y
+    coordinates. The parameter can assume one of three values:
+
+    * ``before``: (default) Draw step levels before each x-coordinate (no step before the first point)
+    * ``after``:  Draw step levels after each x-coordinate (no step after the last point)
+    * ``center``: Draw step levels centered on each x-coordinate
+    """)
+
+class Text(XYGlyph):
+    ''' Render text.
+
+    '''
+
+    __example__ = "examples/reference/models/Text.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -918,9 +1019,11 @@ class Text(Glyph):
     """)
 
 class VBar(Glyph):
-    """ Render vertical bars, given a center coordinate, width and (top, bottom) coordinates. """
+    ''' Render vertical bars, given a center coordinate, width and (top, bottom) coordinates.
 
-    __example__ = "tests/glyphs/VBar.py"
+    '''
+
+    __example__ = "examples/reference/models/VBar.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -950,11 +1053,12 @@ class VBar(Glyph):
     The %s values for the vertical bars.
     """)
 
-class Wedge(Glyph):
-    """ Render wedges.
-    """
+class Wedge(XYGlyph):
+    ''' Render wedges.
 
-    __example__ = "tests/glyphs/Wedge.py"
+    '''
+
+    __example__ = "examples/reference/models/Wedge.py"
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
@@ -993,10 +1097,9 @@ class Wedge(Glyph):
     """)
 
 # XXX: allow `from bokeh.models.glyphs import *`
-from .markers import (Marker, Asterisk, Circle, CircleCross, CircleX, Cross,
-                      Diamond, DiamondCross, InvertedTriangle, Square,
-                      SquareCross, SquareX, Triangle, X)
+from .markers import (Asterisk, Circle, CircleCross, CircleX, Cross, Diamond, DiamondCross,
+                      Hex, InvertedTriangle, Marker, Square, SquareCross, SquareX, Triangle, X)
 
 # Fool pyflakes
-(Marker, Asterisk, Circle, CircleCross, CircleX, Cross, Diamond, DiamondCross,
-InvertedTriangle, Square, SquareCross, SquareX, Triangle, X)
+(Asterisk, Circle, CircleCross, CircleX, Cross, Diamond, DiamondCross,
+Hex, InvertedTriangle, Marker, Square, SquareCross, SquareX, Triangle, X)
