@@ -4,29 +4,16 @@ from bokeh.layouts import row, widgetbox
 from bokeh.models import Select
 from bokeh.palettes import Spectral5
 from bokeh.plotting import curdoc, figure
-from bokeh.sampledata.autompg import autompg
+from bokeh.sampledata.autompg import autompg_clean as df
 
-df = autompg.copy()
+df = df.copy()
 
 SIZES = list(range(6, 22, 3))
 COLORS = Spectral5
-ORIGINS = ['North America', 'Europe', 'Asia']
 
 # data cleanup
-df.cyl = [str(x) for x in df.cyl]
-df.origin = [ORIGINS[x-1] for x in df.origin]
-
-df['year'] = [str(x) for x in df.yr]
-del df['yr']
-
-df['mfr'] = [x.split()[0] for x in df.name]
-df.loc[df.mfr=='chevy', 'mfr'] = 'chevrolet'
-df.loc[df.mfr=='chevroelt', 'mfr'] = 'chevrolet'
-df.loc[df.mfr=='maxda', 'mfr'] = 'mazda'
-df.loc[df.mfr=='mercedes-benz', 'mfr'] = 'mercedes'
-df.loc[df.mfr=='toyouta', 'mfr'] = 'toyota'
-df.loc[df.mfr=='vokswagen', 'mfr'] = 'volkswagen'
-df.loc[df.mfr=='vw', 'mfr'] = 'volkswagen'
+df.cyl = df.cyl.astype(str)
+df.yr = df.yr.astype(str)
 del df['name']
 
 columns = sorted(df.columns)
@@ -34,7 +21,6 @@ discrete = [x for x in columns if df[x].dtype == object]
 continuous = [x for x in columns if x not in discrete]
 quantileable = [x for x in continuous if len(df[x].unique()) > 20]
 
-
 def create_figure():
     xs = df[x.value].values
     ys = df[y.value].values
