@@ -52,7 +52,7 @@ def create_plot(foos, title='', r = 1, y_range=None, period = pi/2, cfoos=None):
         y_range=[-2, 2]
 
     # create new figure
-    p = figure(title=title, width=800, height=300, x_range=[-2, 9], y_range=y_range)
+    p = figure(title=title, plot_width=800, plot_height=300, x_range=[-2, 9], y_range=y_range)
     p.xgrid.bounds = (-2, 2)
     p.xaxis.bounds = (-2, 2)
 
@@ -127,7 +127,7 @@ def update_centric_sources(sources, foos, newx, ind, cfoos):
         get_new_sources(newx, foo, sources[i], cfoos[i])
 
 def create_centric_plot(foos, title='', r = 1, y_range=(-2, 2), period = pi/2, cfoos=None):
-    p = figure(title=title, width=800, height=300, x_range=[-2, 9], y_range=y_range)
+    p = figure(title=title, plot_width=800, plot_height=300, x_range=[-2, 9], y_range=y_range)
     p.xgrid.bounds = (-2, 2)
     p.xaxis.bounds = (-2, 2)
 
@@ -150,9 +150,9 @@ def create_centric_plot(foos, title='', r = 1, y_range=(-2, 2), period = pi/2, c
 
     p.legend.location = "top_right"
     p.legend.orientation = "horizontal"
-    p.legend.legend_padding = 6
-    p.legend.legend_margin = 6
-    p.legend.legend_spacing = 6
+    p.legend.padding = 6
+    p.legend.margin = 6
+    p.legend.spacing = 6
 
     return p, _sources
 
