@@ -1,3 +1,4 @@
+import io
 
 from jinja2 import Template
 
@@ -8,7 +9,7 @@ from bokeh.resources import INLINE
 from bokeh.util.browser import view
 
 # create some data
-x1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
+x1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12]
 y1 = [0, 8, 2, 4, 6, 9, 5, 6, 25, 28, 4, 7]
 x2 = [2, 5, 7, 15, 18, 19, 25, 28, 9, 10, 4]
 y2 = [2, 4, 6, 9, 15, 18, 0, 8, 2, 25, 28]
@@ -49,10 +50,19 @@ template = Template('''<!DOCTYPE html>
         {{ js_resources }}
         {{ css_resources }}
         {{ script }}
+        <style>
+            .embed-wrapper {
+                width: 50%;
+                height: 400px;
+                margin: auto;
+            }
+        </style>
     </head>
     <body>
         {% for key in div.keys() %}
+            <div class="embed-wrapper">
             {{ div[key] }}
+            </div>
         {% endfor %}
     </body>
 </html>
@@ -68,7 +78,7 @@ html = template.render(js_resources=js_resources,
                        script=script,
                        div=div)
 
-with open(filename, 'w') as f:
+with io.open(filename, mode='w', encoding='utf-8') as f:
     f.write(html)
 
 view(filename)
