@@ -61,18 +61,21 @@ callback = CustomJS(args=dict(source=source), code="""
     function rgbToHex(r, g, b) {
         return "#" + componentToHex(r) + componentToHex(g) + componentToHex(b);
     }
+    function toInt(v) {
+       return v | 0;
+    }
     var data = source.data;
     color = data['color'];
     text_color = data['text_color'];
-    var R = red_slider.value;
-    var G = green_slider.value;
-    var B = blue_slider.value;
+    var R = toInt(red_slider.value);
+    var G = toInt(green_slider.value);
+    var B = toInt(blue_slider.value);
     color[0] = rgbToHex(R, G, B);
     text_color[0] = '#ffffff';
     if ((R > 127) || (G > 127) || (B > 127)) {
         text_color[0] = '#000000';
     }
-    source.trigger('change');
+    source.change.emit();
 """)
 
 # create slider tool objects with a callback to control the RGB levels for first plot
