@@ -1,9 +1,17 @@
-from bokeh.models import HoverTool
-from bokeh.palettes import Viridis6
-from bokeh.plotting import figure, show, output_file, ColumnDataSource
+from bokeh.io import show
+from bokeh.models import (
+    ColumnDataSource,
+    HoverTool,
+    LogColorMapper
+)
+from bokeh.palettes import Viridis6 as palette
+from bokeh.plotting import figure
+
 from bokeh.sampledata.us_counties import data as counties
 from bokeh.sampledata.unemployment import data as unemployment
 
+palette.reverse()
+
 counties = {
     code: county for code, county in counties.items() if county["state"] == "tx"
 }
@@ -13,25 +21,26 @@ county_ys = [county["lats"] for county in counties.values()]
 
 county_names = [county['name'] for county in counties.values()]
 county_rates = [unemployment[county_id] for county_id in counties]
-county_colors = [Viridis6[int(rate/3)] for rate in county_rates]
+color_mapper = LogColorMapper(palette=palette)
 
 source = ColumnDataSource(data=dict(
     x=county_xs,
     y=county_ys,
-    color=county_colors,
     name=county_names,
     rate=county_rates,
 ))
 
-TOOLS="pan,wheel_zoom,box_zoom,reset,hover,save"
+TOOLS = "pan,wheel_zoom,reset,hover,save"
 
-p = figure(title="Texas Unemployment 2009", tools=TOOLS,
-           x_axis_location=None, y_axis_location=None)
+p = figure(
+    title="Texas Unemployment, 2009", tools=TOOLS,
+    x_axis_location=None, y_axis_location=None
+)
 p.grid.grid_line_color = None
 
 p.patches('x', 'y', source=source,
-          fill_color='color', fill_alpha=0.7,
-          line_color="white", line_width=0.5)
+          fill_color={'field': 'rate', 'transform': color_mapper},
+          fill_alpha=0.7, line_color="white", line_width=0.5)
 
 hover = p.select_one(HoverTool)
 hover.point_policy = "follow_mouse"
@@ -41,6 +50,4 @@ hover.tooltips = [
     ("(Long, Lat)", "($x, $y)"),
 ]
 
-output_file("texas.html", title="texas.py example")
-
 show(p)
